'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.OrderHandler = exports.Item = exports.Gridifier = undefined;

var _item = require('./item');

Object.defineProperty(exports, 'Item', {
  enumerable: true,
  get: function () {
    return _interopRequireDefault(_item).default;
  }
});

var _orderHandler = require('./order-handler');

Object.defineProperty(exports, 'OrderHandler', {
  enumerable: true,
  get: function () {
    return _interopRequireDefault(_orderHandler).default;
  }
});

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _gridifier = require('gridifier');

var _gridifier2 = _interopRequireDefault(_gridifier);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Gridifier extends _react2.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      gridSettings: this.gridSettingsFromProps(this.props)
    };
  }

  componentDidMount() {
    this._gridDOMNode = _reactDom2.default.findDOMNode(this.refs.grid);
    this._grid = new _gridifier2.default(this._gridDOMNode, this.state.gridSettings);

    this.updateOrderHandler(this.props.orderHandler, null);

    this.updateEditHandlersVisibility();
    this.connectAddedChildren();
  }

  componentWillReceiveProps(nextProps) {}

  shouldComponentUpdate(nextProps) {
    console.log('###$$$', nextProps.editable !== this.props.editable, nextProps.children.length !== this.props.children.length, nextProps.orderHandler !== this.props.orderHandler);
    return nextProps.editable !== this.props.editable;
  }

  componentWillUpdate(nextProps, nextState) {
    this.disconnectRemovedChildren(nextProps.children);

    const nextGridSettings = this.gridSettingsFromProps(nextProps);
    this.updateGridSettings(nextGridSettings, nextState);
  }

  componentDidUpdate(nextProps, nextState) {
    this.updateEditHandlersVisibility();
    this.connectAddedChildren();
  }

  componentWillUnmount() {
    this._grid.destroy();
    delete this._grid;
  }

  render() {
    const className = (0, _classnames2.default)(this.props.itemBackgrounds ? 'rg-item-backgrounds rg-grid' : 'rg-grid', this.props.className);
    return _react2.default.createElement(
      'div',
      { ref: 'grid', className: className },
      this.props.children
    );
  }

  gridSettingsFromProps(nextProps) {
    const settings = {
      class: 'rg-grid-item',
      grid: nextProps.grid || 'vertical',
      prepend: nextProps.prepend || 'mirrored',
      append: nextProps.append || 'default',
      intersections: nextProps.intersections,
      sortDispersion: nextProps.sortDispersion,
      loadImages: nextProps.loadImages,
      toggleTime: nextProps.toggleTime || 0,
      coordsChangeTime: nextProps.coordsChangeTime || 0,
      toggleTiming: nextProps.toggleTiming,
      coordsChangeTiming: nextProps.coordsChangeTiming,
      rotatePerspective: nextProps.rotatePerspective,
      rotateBackface: nextProps.rotateBackface,
      rotateAngles: nextProps.rotateAngles,
      gridResize: nextProps.gridResize || 'fit',
      gridResizeDelay: nextProps.gridResizeDelay || 0,
      dragifier: 'rg-drag-handler',
      dragifierMode: nextProps.dragifierMode || 'i',
      widthPtAs: 0,
      heightPtAs: 0,
      widthPxAs: 0,
      heightPxAs: 0,
      vpResizeDelay: 10,
      queueSize: nextProps.queueSize || 12,
      queueDelay: nextProps.queueDelay || 25,

      // above does not belongs to gridifier, but used anyway by react component
      editable: nextProps.editable,
      orderHandler: nextProps.orderHandler
    };
    if (nextProps.align) settings.align = nextProps.align; // null|undefined not working. Key must be omitted in the case

    return settings;
  }

  updateGridSettings(nextGridSettings, nextState) {
    for (let k in nextGridSettings) {
      if (!nextGridSettings.hasOwnProperty(k)) {
        continue;
      }
      const v = nextGridSettings[k];
      if (nextState.gridSettings && nextState.gridSettings[k] === v) {
        continue;
      }

      switch (k) {
        case 'editable':
          v ? this._grid.dragifierOn() : this._grid.dragifierOff();
          break;
        case 'orderHandler':
          this.updateOrderHandler(v, nextState.gridSettings[k]);
          break;
        default:
          this._grid.set(k, v).reposition();
      }
    }
  }

  connectAddedChildren() {
    if (this.props.insertionMode === 'prepend') {
      this._grid.prependNew();
    } else {
      this._grid.appendNew();
    }
  }

  disconnectRemovedChildren(nextChildren) {
    if (!nextChildren) {
      return;
    }
    const nextChildrenIds = nextChildren.map(child => child.props.id);
    const children = this._grid.collectConnected();
    children.forEach(child => {
      if (!nextChildrenIds.includes(child.id)) {
        this._grid.disconnect(child);
      }
    });
  }

  updateEditHandlersVisibility() {
    [].forEach.call(document.getElementsByClassName('rg-edition-tool'), element => {
      element.style.display = this.props.editable ? 'flex' : 'none';
    });
  }

  updateOrderHandler(addHandler, removeHandler) {
    if (removeHandler) {
      removeHandler.removeAllListeners('sort-order-list');
      // FIXME: unplug handler listening on grid events: problem, there is no func to do this in v.2.0.3!
    }
    if (addHandler && addHandler.onChange) {
      const that = this;
      addHandler.on('sort-order-list', list => {
        list = [].slice.call(list);
        if (list.length > 0) {
          that._fromIdList = list.map(item => item.id);
          that._grid.addApi('sort', 'fromIdList', ((first, second) => {
            return this._fromIdList.indexOf(first.id) - this._fromIdList.indexOf(second.id);
          }).bind(that)); // eslint-disable-line no-extra-bind
          that._grid.sort('fromIdList').resort();
        }
      });
      addHandler.restoreOrder(); // init is run before plugin grid events

      this._grid.onGridResize(addHandler.onChange.bind(addHandler));
    }
  }

  reposition() {
    this._grid.reposition();
  }
}

Gridifier.propTypes = {
  editable: _propTypes2.default.bool.isRequired,
  orderHandler: _propTypes2.default.object,
  insertionMode: _propTypes2.default.oneOf(['append', 'prepend']).isRequired,
  itemBackgrounds: _propTypes2.default.bool.isRequired,
  grid: _propTypes2.default.oneOf(['vertical', 'horizontal']),
  prepend: _propTypes2.default.oneOf(['mirrored', 'default', 'reversed']),
  append: _propTypes2.default.oneOf(['default', 'reversed']),
  intersections: _propTypes2.default.bool.isRequired,
  align: _propTypes2.default.oneOf(['top', 'center', 'bottom', 'left', 'right']),
  sortDispersion: _propTypes2.default.bool.isRequired,
  loadImages: _propTypes2.default.bool.isRequired,
  toggleTime: _propTypes2.default.number,
  coordsChangeTime: _propTypes2.default.number,
  toggleTiming: _propTypes2.default.string,
  coordsChangeTiming: _propTypes2.default.string,
  rotatePerspective: _propTypes2.default.string,
  rotateBackface: _propTypes2.default.bool.isRequired,
  rotateAngles: _propTypes2.default.array,
  gridResize: _propTypes2.default.oneOf(['fit', 'expand', 'disabled']),
  gridResizeDelay: _propTypes2.default.number,
  dragifierMode: _propTypes2.default.oneOf(['i', 'd']),
  queueSize: _propTypes2.default.number,
  queueDelay: _propTypes2.default.number
};

Gridifier.defaultProps = {
  editable: false,
  insertionMode: 'append',
  itemBackgrounds: false,
  intersections: true,
  sortDispersion: false,
  loadImages: false,
  toggleTiming: 'ease',
  coordsChangeTiming: 'ease',
  rotatePerspective: '200px',
  rotateBackface: true,
  rotateAngles: [0, -180, 180, 0]
};

exports.default = Gridifier;
exports.Gridifier = Gridifier;